searchNodes=[{"ref":"Delx.html","title":"Delx","type":"module","doc":"An Elixir library to make function delegation testable. Usage Let&#39;s say you have the following module. iex&gt; defmodule Greeter.StringGreeter do ...&gt; def hello(name) do ...&gt; &quot;Hello, \#{name}!&quot; ...&gt; end ...&gt; end You can delegate functions calls to another module by using the Delx module and calling the defdelegate/2 macro in the module body. It has the same API as Elixir&#39;s own Kernel.defdelegate/2 macro. iex&gt; defmodule Greeter do ...&gt; use Delx, otp_app: :greeter ...&gt; defdelegate hello(name), to: Greeter.StringGreeter ...&gt; end iex&gt; Greeter.hello(&quot;Tobi&quot;) &quot;Hello, Tobi!&quot; The reason you need to use Delx and define the :otp_app option is that each application can configure their own delegation behavior. So third-party libaries that also use Delx remain unaffected of your application-specific configuration. Testing One great benefit of Delx is that you can test delegation without invoking the actual implementation of the delegation target, thus eliminating all side effects. Built-In Assertions Delx brings it&#39;s own test assertions. All you need to do is to activate delegation mocking for your test environment by putting the following line in your config/test.exs: config :greeter, Delx, mock: true Then in your tests, you can import Delx.TestAssertions and use the Delx.TestAssertions.assert_delegate/2 and Delx.TestAssertions.refute_delegate/2 assertions. defmodule GreeterTest do use ExUnit.Case import Delx.TestAssertions describe &quot;hello/1&quot; do test &quot;delegate to Greeter.StringGreeter&quot; do assert_delegate {Greeter, :hello, 1}, to: Greeter.StringGreeter end end end Note that once you activate mocking all delegated functions do not return anymore but instead raise the Delx.MockedDelegationError. If you really want to call the original implementation, you have to avoid any calls of delegated functions. With Mox If you are using Mox in your application you have another possibility to test delegated functions. Register a mock for the Delx.Delegator behavior to your test/test_helper.exs (or wherever you define your mocks): Mox.defmock(Greeter.DelegatorMock, for: Delx.Delegator) Then, in your config/test.exs you have to set the mock as delegator module for your app. config :my_app, Delx, delegator: Greeter.DelegatorMock Please make sure not to use the :mock option and a :delegator option at the same time as this may lead to unexpected behavior. Now you are able to expect calls to delegated functions: defmodule GreeterTest do use ExUnit.Case import Mox setup :verify_on_exit! describe &quot;hello/1&quot; do test &quot;delegate to Greeter.StringGreeter&quot; do expect( Greeter.DelegatorMock, :apply, fn {Greeter, :hello}, {Greeter.StringGreeter, :hello}, [&quot;Tobi&quot;] -&gt; :ok end ) Greeter.hello(&quot;Tobi&quot;) end end end For more information on how to implement your own delegator, refer to the docs of the Delx.Delegator behavior. Note that the configuration is only applied at compile time, so you are unable to mock or replace the delegator module at runtime."},{"ref":"Delx.Defdelegate.html","title":"Delx.Defdelegate","type":"module","doc":"A module defining a macro to define delegate functions."},{"ref":"Delx.Defdelegate.html#defdelegate/2","title":"Delx.Defdelegate.defdelegate/2","type":"macro","doc":"You can delegate functions calls to another module by using the Delx module and calling the defdelegate/2 macro in the module body. It has the same API as Elixir&#39;s own Kernel.defdelegate/2 macro. Example Before calling defdelegate/2, you need to use Delx. iex&gt; defmodule Greeter do ...&gt; use Delx, otp_app: :greeter ...&gt; defdelegate hello(name), to: Greeter.StringGreeter, as: :welcome ...&gt; end"},{"ref":"Delx.MockedDelegationError.html","title":"Delx.MockedDelegationError","type":"exception","doc":"An exception that raises when when a delegated function is mocked."},{"ref":"Delx.TestAssertions.html","title":"Delx.TestAssertions","type":"module","doc":"A module that contains assertions for ExUnit to test function delegation. Note that you need to activate mock mode for your test environment in order to make the assertions work. In your config/test.exs file: config :my_app, Delx, mock: true"},{"ref":"Delx.TestAssertions.html#assert_delegate/2","title":"Delx.TestAssertions.assert_delegate/2","type":"function","doc":"Asserts whether the function specified by MFA (module-function-arity tuple) is delegated to the the given target module. Accepts the same options as the Kernel.defdelegate/2 macro. Options :to - The module to which the function delegates to. :as - The name of the function in the target module. Example defmodule GreeterTest do use ExUnit.Case import Delx.TestAssertions describe &quot;hello/1&quot; do test &quot;delegate to Greeter.StringGreeter&quot; do assert_delegate {Greeter, :hello, 1}, to: Greeter.StringGreeter end end end"},{"ref":"Delx.TestAssertions.html#refute_delegate/2","title":"Delx.TestAssertions.refute_delegate/2","type":"function","doc":"Refutes whether the function specified by MFA (module-function-arity tuple) is delegated to the the given target module. Accepts the same options as the Kernel.defdelegate/2 macro. Options :to - The module to which the function delegates to. :as - The name of the function in the target module. Example defmodule GreeterTest do use ExUnit.Case import Delx.TestAssertions describe &quot;hello/1&quot; do test &quot;delegate to Greeter.StringGreeter&quot; do refute_delegate {Greeter, :hello, 1}, to: Greeter.StringGreeter end end end"},{"ref":"Delx.Delegator.html","title":"Delx.Delegator","type":"behaviour","doc":"A module that defines the function that a custom delegator module has to implement. A delegator only contains an apply/3 function that takes the delegation source as first argument, delegation target as second argument and the forwarded arguments list as third argument."},{"ref":"Delx.Delegator.html#c:apply/3","title":"Delx.Delegator.apply/3","type":"callback","doc":"A callback that needs to be implemented by a delegator."},{"ref":"Delx.Delegator.html#t:mf/0","title":"Delx.Delegator.mf/0","type":"type","doc":"A tuple that contains a module and function name as atom."},{"ref":"Delx.Delegator.Common.html","title":"Delx.Delegator.Common","type":"module","doc":"This module is the delegator that is used by default."},{"ref":"Delx.Delegator.Mock.html","title":"Delx.Delegator.Mock","type":"module","doc":"This module is a custom delegator that does not actually delegate but instead raises an exception containing the delegation details. This is useful when you want to mock all delegations but not actually call the delegation target. This delegator must be used to make Delx.TestAssertions work."},{"ref":"readme.html","title":"Delx","type":"extras","doc":"Delx Defdelegate on steroids! An Elixir library to make function delegation testable."},{"ref":"readme.html#prerequisites","title":"Delx - Prerequisites","type":"extras","doc":"Erlang 20 or greater Elixir 1.8 or greater"},{"ref":"readme.html#installation","title":"Delx - Installation","type":"extras","doc":"If available in Hex, the package can be installed by adding delx to your list of dependencies in mix.exs: def deps do [ {:delx, &quot;~&gt; 2.1&quot;} ] end"},{"ref":"readme.html#usage","title":"Delx - Usage","type":"extras","doc":"Check out the full docs at https://hexdocs.pm/delx. Let&#39;s say you have the following module. defmodule Greeter.StringGreeter do def hello(name) do &quot;Hello, \#{name}!&quot; end end You can delegate functions calls to another module by using the Delx module and calling the defdelegate/2 macro in the module body. It has the same API as Elixir&#39;s own Kernel.defdelegate/2 macro. defmodule Greeter do use Delx, otp_app: :greeter defdelegate hello(name), to: Greeter.StringGreeter end Greeter.hello(&quot;Tobi&quot;) # =&gt; &quot;Hello, Tobi!&quot;"},{"ref":"readme.html#testing","title":"Delx - Testing","type":"extras","doc":"One great benefit of Delx is that you can test delegation without invoking the actual implementation of the delegation target, thus eliminating all side effects. Built-In Assertions Delx brings it&#39;s own test assertions. All you need to do is to activate delegation mocking for your test environment by putting the following line in your config/test.exs: config :greeter, Delx, mock: true Then in your tests, you can import Delx.TestAssertions and use the assert_delegate/2 and refute_delegate/2 assertions. defmodule GreeterTest do use ExUnit.Case import Delx.TestAssertions describe &quot;hello/1&quot; do test &quot;delegate to Greeter.StringGreeter&quot; do assert_delegate {Greeter, :hello, 1}, to: Greeter.StringGreeter end end end Note that once you activate mocking all delegated functions do not return anymore but instead raise the Delx.MockedDelegationError. If you really want to call the original implementation, you have to avoid any calls of delegated functions. With Mox If you are using Mox in your application you have another possibility to test delegated functions. Register a mock for the Delx.Delegator behavior to your test/test_helper.exs (or wherever you define your mocks): Mox.defmock(Greeter.DelegatorMock, for: Delx.Delegator) Then, in your config/test.exs you have to set the mock as delegator module for your app. config :my_app, Delx, delegator: Greeter.DelegatorMock Please make sure not to use the :mock option and a :delegator option at the same time as this may lead to unexpected behavior. Now you are able to expect calls to delegated functions: defmodule GreeterTest do use ExUnit.Case import Mox setup :verify_on_exit! describe &quot;hello/1&quot; do test &quot;delegate to Greeter.StringGreeter&quot; do expect( Greeter.DelegatorMock, :apply, fn {Greeter, :hello}, {Greeter.StringGreeter, :hello}, [&quot;Tobi&quot;] -&gt; :ok end ) Greeter.hello(&quot;Tobi&quot;) end end end For more information on how to implement your own delegator, refer to the docs of the Delx.Delegator behavior. Note that the configuration is only applied at compile time, so you are unable to mock or replace the delegator module at runtime."},{"ref":"readme.html#migration-from-v2-to-v3","title":"Delx - Migration from v2 to v3","type":"extras","doc":"Note that the function defdel/2 has been removed in favor of defdelegate/2 to give a better experience with the tooling. Additionally, let&#39;s say you want to migrate away from Delx you can simply unuse Delx to fall back to the default Kernel.defdelegate/2 and your code will still work. The config option to enable testing mode has changed from stub to mock. If you got the following line in your config: config :greeter, Delx, stub: true Please change it to: config :greeter, Delx, mock: true"}]